@@ -10,6 +10,7 @@ import (
 type Shell interface {
 	Hook() (string, error)
 	Export(e ShellExport) string
+	Dump(env Env) string
 }
 
 // Used to describe what to generate for the shell
@@ -37,12 +38,16 @@ func DetectShell(target string) Shell {
 		return ZSH
 	case "fish":
 		return FISH
+	case "gzenv":
+		return GZENV
 	case "vim":
 		return VIM
 	case "tcsh":
 		return TCSH
 	case "json":
 		return JSON
+	case "elvish":
+		return ELVISH
 	}
 
 	return nil
