@@ -4,11 +4,24 @@ const STDLIB = "#!bash\n" +
 	"#\n" +
 	"# These are the commands available in an .envrc context\n" +
 	"#\n" +
+	"# ShellCheck exceptions:\n" +
+	"#\n" +
+	"# SC1090: Can't follow non-constant source. Use a directive to specify location.\n" +
+	"# SC1091: Not following: (file missing)\n" +
+	"# SC1117: Backslash is literal in \"\\n\". Prefer explicit escaping: \"\\\\n\".\n" +
+	"# SC2059: Don't use variables in the printf format string. Use printf \"..%s..\" \"$foo\".\n" +
 	"set -e\n" +
-	"direnv=\"%s\"\n" +
+	"\n" +
+	"# NOTE: don't touch the RHS, it gets replaced at runtime\n" +
+	"direnv=\"$(command -v direnv)\"\n" +
 	"\n" +
 	"# Config, change in the direnvrc\n" +
-	"DIRENV_LOG_FORMAT=\"${DIRENV_LOG_FORMAT-direnv: %%s}\"\n" +
+	"DIRENV_LOG_FORMAT=\"${DIRENV_LOG_FORMAT-direnv: %s}\"\n" +
+	"\n" +
+	"# This variable can be used by programs to detect when they are running inside\n" +
+	"# of a .envrc evaluation context. All variables starting with `DIRENV_` are\n" +
+	"# ignored by the direnv diffing algorithm and so it won't be re-exported.\n" +
+	"export DIRENV_IN_ENVRC=1\n" +
 	"\n" +
 	"# Usage: direnv_layout_dir\n" +
 	"#\n" +
@@ -35,7 +48,7 @@ const STDLIB = "#!bash\n" +
 	"log_status() {\n" +
 	"  if [[ -n $DIRENV_LOG_FORMAT ]]; then\n" +
 	"    local msg=$*\n" +
-	"    # shellcheck disable=SC2059\n" +
+	"    # shellcheck disable=SC2059,SC1117\n" +
 	"    printf \"${DIRENV_LOG_FORMAT}\\n\" \"$msg\" >&2\n" +
 	"  fi\n" +
 	"}\n" +
@@ -58,7 +71,7 @@ const STDLIB = "#!bash\n" +
 	"  color_error=$(tput setaf 1)\n" +
 	"  if [[ -n $DIRENV_LOG_FORMAT ]]; then\n" +
 	"    local msg=$*\n" +
-	"    # shellcheck disable=SC2059\n" +
+	"    # shellcheck disable=SC2059,SC1117\n" +
 	"    printf \"${color_error}${DIRENV_LOG_FORMAT}${color_normal}\\n\" \"$msg\" >&2\n" +
 	"  fi\n" +
 	"}\n" +
@@ -78,6 +91,15 @@ const STDLIB = "#!bash\n" +
 	"  type \"$1\" &>/dev/null\n" +
 	"}\n" +
 	"\n" +
+	"# Usage: join_args [args...]\n" +
+	"#\n" +
+	"# Joins all the passed arguments into a single string that can be evaluated by bash\n" +
+	"#\n" +
+	"# This is useful when one has to serialize an array of arguments back into a string\n" +
+	"join_args() {\n" +
+	"  printf '%q ' \"$@\"\n" +
+	"}\n" +
+	"\n" +
 	"# Usage: expand_path <rel_path> [<relative_to>]\n" +
 	"#\n" +
 	"# Outputs the absolute path of <rel_path> relative to <relative_to> or the\n" +
@@ -162,7 +184,7 @@ const STDLIB = "#!bash\n" +
 	"        echo \"$PWD/$1\"\n" +
 	"        return 0\n" +
 	"      fi\n" +
-	"      if [[ $PWD = / ]] || [[ $PWD = // ]]; then\n" +
+	"      if [[ $PWD == / ]] || [[ $PWD == // ]]; then\n" +
 	"        return 1\n" +
 	"      fi\n" +
 	"      cd ..\n" +
@@ -185,17 +207,17 @@ const STDLIB = "#!bash\n" +
 	"  rcfile=$(user_rel_path \"$rcpath\")\n" +
 	"  watch_file \"$rcpath\"\n" +
 	"\n" +
-	"  pushd \"$(pwd 2>/dev/null)\" > /dev/null\n" +
-	"    pushd \"$(dirname \"$rcpath\")\" > /dev/null\n" +
-	"    if [[ -f ./$(basename \"$rcpath\") ]]; then\n" +
-	"      log_status \"loading $rcfile\"\n" +
-	"      # shellcheck source=/dev/null\n" +
-	"      . \"./$(basename \"$rcpath\")\"\n" +
-	"    else\n" +
-	"      log_status \"referenced $rcfile does not exist\"\n" +
-	"    fi\n" +
-	"    popd > /dev/null\n" +
-	"  popd > /dev/null\n" +
+	"  pushd \"$(pwd 2>/dev/null)\" >/dev/null\n" +
+	"  pushd \"$(dirname \"$rcpath\")\" >/dev/null\n" +
+	"  if [[ -f ./$(basename \"$rcpath\") ]]; then\n" +
+	"    log_status \"loading $rcfile\"\n" +
+	"    # shellcheck source=/dev/null\n" +
+	"    . \"./$(basename \"$rcpath\")\"\n" +
+	"  else\n" +
+	"    log_status \"referenced $rcfile does not exist\"\n" +
+	"  fi\n" +
+	"  popd >/dev/null\n" +
+	"  popd >/dev/null\n" +
 	"}\n" +
 	"\n" +
 	"# Usage: watch_file <filename>\n" +
@@ -209,7 +231,6 @@ const STDLIB = "#!bash\n" +
 	"  eval \"$(\"$direnv\" watch \"$file\")\"\n" +
 	"}\n" +
 	"\n" +
-	"\n" +
 	"# Usage: source_up [<filename>]\n" +
 	"#\n" +
 	"# Loads another \".envrc\" if found with the find_up command.\n" +
@@ -228,7 +249,7 @@ const STDLIB = "#!bash\n" +
 	"}\n" +
 	"\n" +
 	"# Usage: direnv_load <command-generating-dump-output>\n" +
-	"#   e.g: direnv_load opam-env exec -- direnv dump\n" +
+	"#   e.g: direnv_load opam-env exec -- \"$direnv\" dump\n" +
 	"#\n" +
 	"# Applies the environment generated by running <argv> as a\n" +
 	"# command. This is useful for adopting the environment of a child\n" +
@@ -237,12 +258,17 @@ const STDLIB = "#!bash\n" +
 	"#\n" +
 	"direnv_load() {\n" +
 	"  local exports\n" +
+	"  # backup and restore watches in case of nix-shell --pure\n" +
+	"  local __watches=$DIRENV_WATCHES\n" +
+	"\n" +
 	"  exports=$(\"$direnv\" apply_dump <(\"$@\"))\n" +
 	"  local es=$?\n" +
 	"  if [[ $es -ne 0 ]]; then\n" +
 	"    return $es\n" +
 	"  fi\n" +
 	"  eval \"$exports\"\n" +
+	"\n" +
+	"  export DIRENV_WATCHES=$__watches\n" +
 	"}\n" +
 	"\n" +
 	"# Usage: PATH_add <path> [<path> ...]\n" +
@@ -271,19 +297,23 @@ const STDLIB = "#!bash\n" +
 	"#\n" +
 	"# Works like PATH_add except that it's for an arbitrary <varname>.\n" +
 	"path_add() {\n" +
+	"  local path\n" +
 	"  local var_name=\"$1\"\n" +
 	"  # split existing paths into an array\n" +
 	"  declare -a path_array\n" +
-	"  IFS=: read -ra path_array <<< \"${!1}\"\n" +
+	"  IFS=: read -ra path_array <<<\"${!1}\"\n" +
 	"  shift\n" +
 	"\n" +
 	"  # prepend the passed paths in the right order\n" +
-	"  for (( i=$# ; i>0 ; i-- )); do\n" +
-	"    path_array=( \"$(expand_path \"${!i}\")\" \"${path_array[@]}\" )\n" +
+	"  for ((i = $#; i > 0; i--)); do\n" +
+	"    path_array=(\"$(expand_path \"${!i}\")\" \"${path_array[@]}\")\n" +
 	"  done\n" +
 	"\n" +
 	"  # join back all the paths\n" +
-	"  local path=$(IFS=:; echo \"${path_array[*]}\")\n" +
+	"  path=$(\n" +
+	"    IFS=:\n" +
+	"    echo \"${path_array[*]}\"\n" +
+	"  )\n" +
 	"\n" +
 	"  # and finally export back the result to the original variable\n" +
 	"  export \"$var_name=$path\"\n" +
@@ -372,7 +402,8 @@ const STDLIB = "#!bash\n" +
 	"# See http://search.cpan.org/dist/local-lib/lib/local/lib.pm for more details\n" +
 	"#\n" +
 	"layout_perl() {\n" +
-	"  local libdir=$(direnv_layout_dir)/perl5\n" +
+	"  local libdir\n" +
+	"  libdir=$(direnv_layout_dir)/perl5\n" +
 	"  export LOCAL_LIB_DIR=$libdir\n" +
 	"  export PERL_MB_OPT=\"--install_base '$libdir'\"\n" +
 	"  export PERL_MM_OPT=\"INSTALL_BASE=$libdir\"\n" +
@@ -381,6 +412,13 @@ const STDLIB = "#!bash\n" +
 	"  PATH_add \"$libdir/bin\"\n" +
 	"}\n" +
 	"\n" +
+	"# Usage: layout php\n" +
+	"#\n" +
+	"# Adds \"$PWD/vendor/bin\" to the PATH environment variable\n" +
+	"layout_php() {\n" +
+	"  PATH_add vendor/bin\n" +
+	"}\n" +
+	"\n" +
 	"# Usage: layout python <python_exe>\n" +
 	"#\n" +
 	"# Creates and loads a virtualenv environment under\n" +
@@ -391,11 +429,12 @@ const STDLIB = "#!bash\n" +
 	"# versions of python.\n" +
 	"#\n" +
 	"layout_python() {\n" +
+	"  local old_env\n" +
 	"  local python=${1:-python}\n" +
 	"  [[ $# -gt 0 ]] && shift\n" +
-	"  local old_env=$(direnv_layout_dir)/virtualenv\n" +
+	"  old_env=$(direnv_layout_dir)/virtualenv\n" +
 	"  unset PYTHONHOME\n" +
-	"  if [[ -d $old_env && $python = python ]]; then\n" +
+	"  if [[ -d $old_env && $python == python ]]; then\n" +
 	"    export VIRTUAL_ENV=$old_env\n" +
 	"  else\n" +
 	"    local python_version\n" +
@@ -405,7 +444,8 @@ const STDLIB = "#!bash\n" +
 	"      return 1\n" +
 	"    fi\n" +
 	"\n" +
-	"    export VIRTUAL_ENV=$(direnv_layout_dir)/python-$python_version\n" +
+	"    VIRTUAL_ENV=$(direnv_layout_dir)/python-$python_version\n" +
+	"    export VIRTUAL_ENV\n" +
 	"    if [[ ! -d $VIRTUAL_ENV ]]; then\n" +
 	"      virtualenv \"--python=$python\" \"$@\" \"$VIRTUAL_ENV\"\n" +
 	"    fi\n" +
@@ -429,7 +469,7 @@ const STDLIB = "#!bash\n" +
 	"  layout_python python3 \"$@\"\n" +
 	"}\n" +
 	"\n" +
-	"# Usage: layout anaconda <enviroment_name> [<conda_exe>]\n" +
+	"# Usage: layout anaconda <environment_name> [<conda_exe>]\n" +
 	"#\n" +
 	"# Activates anaconda for the named environment. If the environment\n" +
 	"# hasn't been created, it will be using the environment.yml file in\n" +
@@ -438,17 +478,18 @@ const STDLIB = "#!bash\n" +
 	"#\n" +
 	"layout_anaconda() {\n" +
 	"  local env_name=$1\n" +
+	"  local env_loc\n" +
 	"  local conda\n" +
 	"  if [[ $# -gt 1 ]]; then\n" +
 	"    conda=${2}\n" +
 	"  else\n" +
 	"    conda=$(command -v conda)\n" +
 	"  fi\n" +
-	"  PATH_add $(dirname \"$conda\")\n" +
-	"  local env_loc=$(\"$conda\" env list | grep -- \"$env_name\")\n" +
+	"  PATH_add \"$(dirname \"$conda\")\"\n" +
+	"  env_loc=$(\"$conda\" env list | grep -- \"$env_name\")\n" +
 	"  if [[ ! \"$env_loc\" == $env_name*$env_name ]]; then\n" +
 	"    if [[ -e environment.yml ]]; then\n" +
-	"      log_status \"creating conda enviroment\"\n" +
+	"      log_status \"creating conda environment\"\n" +
 	"      \"$conda\" env create\n" +
 	"    else\n" +
 	"      log_error \"Could not find environment.yml\"\n" +
@@ -456,6 +497,7 @@ const STDLIB = "#!bash\n" +
 	"    fi\n" +
 	"  fi\n" +
 	"\n" +
+	"  # shellcheck disable=SC1091\n" +
 	"  source activate \"$env_name\"\n" +
 	"}\n" +
 	"\n" +
@@ -465,19 +507,24 @@ const STDLIB = "#!bash\n" +
 	"# virtualenv from the Pipfile located in the same directory.\n" +
 	"#\n" +
 	"layout_pipenv() {\n" +
-	"  if [[ ! -f Pipfile ]]; then\n" +
-	"    log_error 'No Pipfile found.  Use `pipenv` to create a Pipfile first.'\n" +
+	"  local venv\n" +
+	"  PIPENV_PIPFILE=\"${PIPENV_PIPFILE:-Pipfile}\"\n" +
+	"  if [[ ! -f \"$PIPENV_PIPFILE\" ]]; then\n" +
+	"    log_error \"No Pipfile found.  Use \\`pipenv\\` to create a \\`$PIPENV_PIPFILE\\` first.\"\n" +
 	"    exit 2\n" +
 	"  fi\n" +
 	"\n" +
-	"  local VENV=$(pipenv --bare --venv 2>/dev/null)\n" +
-	"  if [[ -z $VENV || ! -d $VENV ]]; then\n" +
+	"  venv=$(pipenv --bare --venv 2>/dev/null)\n" +
+	"\n" +
+	"  if [[ -z $venv || ! -d $venv ]]; then\n" +
 	"    pipenv install --dev\n" +
 	"  fi\n" +
 	"\n" +
-	"  export VIRTUAL_ENV=$(pipenv --venv)\n" +
+	"  VIRTUAL_ENV=$(pipenv --venv)\n" +
+	"\n" +
 	"  PATH_add \"$VIRTUAL_ENV/bin\"\n" +
 	"  export PIPENV_ACTIVE=1\n" +
+	"  export VIRTUAL_ENV\n" +
 	"}\n" +
 	"\n" +
 	"# Usage: layout ruby\n" +
@@ -488,14 +535,18 @@ const STDLIB = "#!bash\n" +
 	"# directly instead of using the $(bundle exec) prefix.\n" +
 	"#\n" +
 	"layout_ruby() {\n" +
+	"  BUNDLE_BIN=$(direnv_layout_dir)/bin\n" +
+	"\n" +
 	"  if ruby -e \"exit Gem::VERSION > '2.2.0'\" 2>/dev/null; then\n" +
-	"    export GEM_HOME=$(direnv_layout_dir)/ruby\n" +
+	"    GEM_HOME=$(direnv_layout_dir)/ruby\n" +
 	"  else\n" +
 	"    local ruby_version\n" +
 	"    ruby_version=$(ruby -e\"puts (defined?(RUBY_ENGINE) ? RUBY_ENGINE : 'ruby') + '-' + RUBY_VERSION\")\n" +
-	"    export GEM_HOME=$(direnv_layout_dir)/ruby-${ruby_version}\n" +
+	"    GEM_HOME=$(direnv_layout_dir)/ruby-${ruby_version}\n" +
 	"  fi\n" +
-	"  export BUNDLE_BIN=$(direnv_layout_dir)/bin\n" +
+	"\n" +
+	"  export BUNDLE_BIN\n" +
+	"  export GEM_HOME\n" +
 	"\n" +
 	"  PATH_add \"$GEM_HOME/bin\"\n" +
 	"  PATH_add \"$BUNDLE_BIN\"\n" +
@@ -578,12 +629,12 @@ const STDLIB = "#!bash\n" +
 	"  fi\n" +
 	"\n" +
 	"  if [[ -z $version ]] && [[ -f .nvmrc ]]; then\n" +
-	"    version=$(< .nvmrc)\n" +
+	"    version=$(<.nvmrc)\n" +
 	"    via=\".nvmrc\"\n" +
 	"  fi\n" +
 	"\n" +
 	"  if [[ -z $version ]] && [[ -f .node-version ]]; then\n" +
-	"    version=$(< .node-version)\n" +
+	"    version=$(<.node-version)\n" +
 	"    via=\".node-version\"\n" +
 	"  fi\n" +
 	"\n" +
@@ -595,17 +646,14 @@ const STDLIB = "#!bash\n" +
 	"  node_wanted=${node_version_prefix}${version}\n" +
 	"  node_prefix=$(\n" +
 	"    # Look for matching node versions in $NODE_VERSIONS path\n" +
-	"    find \"$NODE_VERSIONS\" -maxdepth 1 -mindepth 1 -type d -name \"$node_wanted*\" |\n" +
-	"\n" +
 	"    # Strip possible \"/\" suffix from $NODE_VERSIONS, then use that to\n" +
 	"    # Strip $NODE_VERSIONS/$NODE_VERSION_PREFIX prefix from line.\n" +
-	"    while IFS= read -r line; do echo \"${line#${NODE_VERSIONS%/}/${node_version_prefix}}\"; done |\n" +
-	"\n" +
 	"    # Sort by version: split by \".\" then reverse numeric sort for each piece of the version string\n" +
-	"    sort -t . -k 1,1rn -k 2,2rn -k 3,3rn |\n" +
-	"\n" +
 	"    # The first one is the highest\n" +
-	"    head -1\n" +
+	"    find \"$NODE_VERSIONS\" -maxdepth 1 -mindepth 1 -type d -name \"$node_wanted*\" \\\n" +
+	"      | while IFS= read -r line; do echo \"${line#${NODE_VERSIONS%/}/${node_version_prefix}}\"; done \\\n" +
+	"      | sort -t . -k 1,1rn -k 2,2rn -k 3,3rn \\\n" +
+	"      | head -1\n" +
 	"  )\n" +
 	"\n" +
 	"  node_prefix=\"${NODE_VERSIONS}/${node_version_prefix}${node_prefix}\"\n" +
@@ -637,20 +685,11 @@ const STDLIB = "#!bash\n" +
 	"# (e.g `use nix -p ocaml`).\n" +
 	"#\n" +
 	"use_nix() {\n" +
-	"  local orig_IN_NIX_SHELL=\"$IN_NIX_SHELL\"\n" +
-	"\n" +
-	"  direnv_load nix-shell --show-trace \"$@\" --run 'direnv dump'\n" +
-	"  if [[ $# = 0 ]]; then\n" +
+	"  direnv_load nix-shell --show-trace \"$@\" --run \"$(join_args \"$direnv\" dump)\"\n" +
+	"  if [[ $# == 0 ]]; then\n" +
 	"    watch_file default.nix\n" +
 	"    watch_file shell.nix\n" +
 	"  fi\n" +
-	"\n" +
-	"  # Don't change the IN_NIX_SHELL env var\n" +
-	"  if [[ -z $orig_IN_NIX_SHELL ]]; then\n" +
-	"    unset IN_NIX_SHELL\n" +
-	"  else\n" +
-	"    export IN_NIX_SHELL=\"$orig_IN_NIX_SHELL\"\n" +
-	"  fi\n" +
 	"}\n" +
 	"\n" +
 	"# Usage: use_guix [...]\n" +
@@ -669,8 +708,10 @@ const STDLIB = "#!bash\n" +
 	"\n" +
 	"## Load the global ~/.direnvrc if present\n" +
 	"if [[ -f ${XDG_CONFIG_HOME:-$HOME/.config}/direnv/direnvrc ]]; then\n" +
+	"  # shellcheck disable=SC1090\n" +
 	"  source \"${XDG_CONFIG_HOME:-$HOME/.config}/direnv/direnvrc\" >&2\n" +
 	"elif [[ -f $HOME/.direnvrc ]]; then\n" +
+	"  # shellcheck disable=SC1090\n" +
 	"  source \"$HOME/.direnvrc\" >&2\n" +
 	"fi\n" +
 	""
